import sofia.micro.*;

//-------------------------------------------------------------------------
/**
 * A wombat, from the tutorial.
 *
 * @author  Michael Kolling
 * @author  Stephen Edwards
 * @author  Last changed by $Author: edwards $
 * @version $Date: 2012/08/23 15:35 $
 */
public class Wombat
    extends Actor
{
    //~ Fields ................................................................

    private static final int EAST = 0;
    private static final int WEST = 1;
    private static final int NORTH = 2;
    private static final int SOUTH = 3;

    private int direction;
    private int leavesEaten;


    //~ Constructor ...........................................................

    // ----------------------------------------------------------
    /**
     * Create a new leaf.
     */
    public Wombat()
    {
        setDirection(EAST);
        leavesEaten = 0;
    }


    // ----------------------------------------------------------
    /**
     * Do whatever the wombat likes to to just now.
     */
    public void act()
    {
        if (foundLeaf())
        {
            eatLeaf();
        }
        else if (canMove())
        {
            move();
        }
        else
        {
            turnLeft();
        }
    }


    // ----------------------------------------------------------
    /**
     * Check whether there is a leaf in the same cell as we are.
     * @return True if there is a leaf in our current cell.
     */
    public boolean foundLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if (leaf != null)
        {
            return true;
        }
        else
        {
            return false;
        }
    }


    // ----------------------------------------------------------
    /**
     * Eat a leaf.
     */
    public void eatLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if (leaf != null)
        {
            // eat the leaf...
            getWorld().remove(leaf);
            leavesEaten = leavesEaten + 1;
        }
    }


    // ----------------------------------------------------------
    /**
     * Move one cell forward in the current direction.
     */
    public void move()
    {
        if (!canMove())
        {
            return;
        }
        switch (direction)
        {
            case SOUTH :
                setGridY(getGridY() + 1);
                break;
            case EAST :
                setGridX(getGridX() + 1);
                break;
            case NORTH :
                setGridY(getGridY() - 1);
                break;
            case WEST :
                setGridX(getGridX() - 1);
                break;
        }
    }


    // ----------------------------------------------------------
    /**
     * Test if we can move forward. Return true if we can, false otherwise.
     * @return True if we can move forward.
     */
    public boolean canMove()
    {
        World myWorld = getWorld();
        int x = getGridX();
        int y = getGridY();
        switch (direction)
        {
            case SOUTH :
                y++;
                break;
            case EAST :
                x++;
                break;
            case NORTH :
                y--;
                break;
            case WEST :
                x--;
                break;
        }
        // test for outside border
        if (x >= myWorld.getWidth() || y >= myWorld.getHeight())
        {
            return false;
        }
        else if (x < 0 || y < 0)
        {
            return false;
        }
        return true;
    }


    // ----------------------------------------------------------
    /**
     * Turns towards the left.
     */
    public void turnLeft()
    {
        switch (direction)
        {
            case SOUTH :
                setDirection(EAST);
                break;
            case EAST :
                setDirection(NORTH);
                break;
            case NORTH :
                setDirection(WEST);
                break;
            case WEST :
                setDirection(SOUTH);
                break;
        }
    }


    // ----------------------------------------------------------
    /**
     * Sets the direction we're facing.
     * @param direction The new direction to face.
     */
    public void setDirection(int direction)
    {
        this.direction = direction;
        switch (direction)
        {
            case SOUTH :
                setRotation(90);
                break;
            case EAST :
                setRotation(0);
                break;
            case NORTH :
                setRotation(270);
                break;
            case WEST :
                setRotation(180);
                break;
            default :
                break;
        }
    }


    // ----------------------------------------------------------
    /**
     * Tell how many leaves we have eaten.
     * @return The number of leaves eaten so far.
     */
    public int getLeavesEaten()
    {
        return leavesEaten;
    }
}
